//
//  OCEActiveComponent.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEActiveComponent.h"
#import "OCEObject.h"
#import "OCENavFrameObject.h"
#import "OCEPathObject.h"

@implementation OCEActiveComponent

- (id)init
{
    [super init];
	[[NSColor whiteColor] retain];

	return self;
}	


- (void)dealloc 
{
    [color release];
    [super dealloc];
}

- (id) initWithRect: (NSRect) aRect
{
	[self init];
	
	xPos = aRect.origin.x;
	yPos = aRect.origin.y;
	width = aRect.size.width;
	height = aRect.size.height;
	
	return self;
}

- (void) setRect: (NSRect) aRect
{
	xPos = aRect.origin.x;
	yPos = aRect.origin.y;
	width = aRect.size.width;
	height = aRect.size.height;
}

- (NSRect) rect
{
	return NSMakeRect(xPos,yPos,width,height);
}

- (void) setColor: (NSColor *) aColor
{
	[color release];
	color = [[aColor colorWithAlphaComponent:0.5] retain];
}

- (void) setFrame: (OCENavFrameObject *) aFrame
{
	frame = aFrame;
}

- (NSColor *) borderColor;
{
	return [color colorWithAlphaComponent:1];
}

- (NSPoint) center
{
	return [frame convertPointFromActiveComponent: NSMakePoint(xPos+width/2,yPos+height/2)]; 
}

- (OCENavFrameObject *) frame
{
	return frame;
}

// ----------------------------------------------------------------------
// Draw
// ----------------------------------------------------------------------
-  (void) drawObjectInRect: (NSRect) aframe isSelected:(bool) selected
{
	NSRect drawRect = NSIntersectionRect(aframe,NSMakeRect(xPos,yPos,width,height));
		
	drawRect.size.width = ((drawRect.size.width == 0) ? 1 : drawRect.size.width);
	drawRect.size.height = ((drawRect.size.height == 0) ? 1 : drawRect.size.height);
	
    NSBezierPath *drawPath=[NSBezierPath bezierPathWithRect:drawRect];
    [color set];
    [drawPath fill];
    
    if (selected)
        [self drawKnobs];
}

- (void) drawKnobs
{
    float length = 4;   
    
    // 1. shadow
    // lower left corner
    NSRect hitRect = NSMakeRect((xPos-length/2.0)+1, (yPos-length/2.0)-1,length,length);
    NSBezierPath *selectPath = [NSBezierPath bezierPathWithRect:hitRect];
    [selectPath setLineWidth:1.0];
    hitRect.origin.y += height; // upper left knob
    [selectPath appendBezierPathWithRect:hitRect];
    hitRect.origin.x += width; // upper right knob
    [selectPath appendBezierPathWithRect:hitRect];
    hitRect.origin.y -= height; //lower right knob
    [selectPath appendBezierPathWithRect:hitRect];
    
    [[NSColor blackColor] set];
    [selectPath fill];
    
    // 2. knob
    NSAffineTransform *knobTransform = [NSAffineTransform transform];
    [knobTransform translateXBy: -1 yBy: 1];
    [knobTransform concat];
    
	[[NSColor knobColor] set];
    
    [selectPath fill];
    [knobTransform invert];
    [knobTransform concat];
}

- (void) drawFocusRing
{
	[frame transformForActiveComponent];
    NSBezierPath *drawPath = [NSBezierPath bezierPathWithRect: NSMakeRect(xPos,yPos,width,height)];
    [[NSColor keyboardFocusIndicatorColor] set];
	[drawPath setLineWidth:2];
    [drawPath stroke];
	
}


- (int) hitTest: (NSPoint)aPoint isSelected: (bool) isSelected
{
    float length = 4+2;   
    
    if (isSelected)
    {
        // lower left knob
        NSRect hitRect = NSMakeRect(floor(1+xPos-length/2.0), floor(-1+yPos-length/2.0),length,length);
        //+1,-1 weil wir den schatten verwenden. dieser gibt die mousposition besser an
        NSBezierPath *cornerPath = [NSBezierPath bezierPathWithRect:hitRect];
        if ([cornerPath containsPoint:aPoint])
            return LowerLeftKnob;
        
        // upper left knob
        hitRect.origin.y += height;
        cornerPath = [NSBezierPath bezierPathWithRect:hitRect];
        if ([cornerPath containsPoint:aPoint])
            return UpperLeftKnob;
        
        // upper right knob
        hitRect.origin.x += width;
        cornerPath = [NSBezierPath bezierPathWithRect:hitRect];
        if ([cornerPath containsPoint:aPoint])
            return UpperRightKnob;
        
        // lower right knob
        hitRect.origin.y -= height;
        cornerPath = [NSBezierPath bezierPathWithRect:hitRect];
        if ([cornerPath containsPoint:aPoint])
            return LowerRightKnob;
    }
    
    // frame hit
    NSBezierPath *framePath = [NSBezierPath bezierPathWithRect:NSMakeRect(xPos,yPos,width,height)];
    if ([framePath containsPoint:aPoint])
    {
        return Object;
    }
    
    return NoKnob; 
}

- (enum corner) movingKnob: (enum corner) knob fromPoint: (NSPoint) lastPoint toPoint: (NSPoint) point 
				 withEvent: (NSEvent *) theEvent

{
	if ((knob == UpperLeftKnob) || (knob == LowerLeftKnob)) 
	{
		// Adjust left edge
		width += xPos - point.x;
		xPos = point.x;
	} 
	else if ((knob == UpperRightKnob) || (knob == LowerRightKnob)) 
	{
		// Adjust right edge
		width = point.x - xPos; 
	}
	if ((knob == UpperLeftKnob) || (knob == UpperRightKnob)) 
	{
		// Adjust top edge
		height = point.y - yPos;
	} 
	else if ((knob == LowerLeftKnob)  || (knob == LowerRightKnob)) 
	{
		// Adjust bottom edge
		height += yPos - point.y;
		yPos = point.y;
	}
	
	// knobs anpassen
	if (width < 0.0) 
	{
		knob = [OCEObject flipKnob:knob horizontal:YES];
		width = abs(width);
		xPos -= width;
	}		
	// knobs anpassen
	if (height < 0.0)
	{
		knob = [OCEObject flipKnob:knob horizontal:NO];
		height = abs(height);
		yPos -= height;
	}
	
	[self adaptPaths];

    return knob;
}

- (void) moveXBy: (double) moveX moveYBy: (double) moveY
{
	xPos += moveX;
	yPos += moveY;
	
	[self adaptPaths];
}

- (bool) adaptToFrame: (NSRect) aFrame
{
	NSRect newRect = NSIntersectionRect(aFrame,NSMakeRect(xPos,yPos,width,height));
	if (NSEqualRects(newRect,NSMakeRect(xPos,yPos,width,height)))
		return YES;
		
	[[undoManager prepareWithInvocationTarget:self] resetWithX: xPos y: yPos width:width height: height];
	xPos = newRect.origin.x;
	yPos = newRect.origin.y;
	width = newRect.size.width;
	height = newRect.size.height;

	if (NSIsEmptyRect(newRect))
		return NO;
	else
		return YES;
}

- (bool) stopManipulationInFrame: (NSRect) aFrame
{
	if (!editingObject)
		return YES;
	
    bool didChange = false;
    if (saveUndoValuesArray[0] != xPos)
        didChange = true;
    
    if (saveUndoValuesArray[1] != yPos)
        didChange = true;
    
    if (saveUndoValuesArray[2] != width)
        didChange = true;
    
    if (saveUndoValuesArray[3] != height)
        didChange = true;

    if (didChange)
    {
		NSRect rect = NSIntersectionRect(aFrame,NSMakeRect(xPos,yPos,width,height));
		
		xPos = rect.origin.x;
		yPos = rect.origin.y;
		width = rect.size.width;
		height = rect.size.height;
		
        [[undoManager prepareWithInvocationTarget:self] resetWithX: saveUndoValuesArray[0] y: saveUndoValuesArray[1] 
                                                             width: saveUndoValuesArray[2] height: saveUndoValuesArray[3]];
		[undoManager setActionName: @"Edit Active Component"];
		if (NSIsEmptyRect(rect))
			return NO;
    }
	editingObject = NO;
	return YES;
}

- (void) resetWithX: (float) x y: (float) y width: (float) w height: (float) h 
{
    // first save old values for redo
    [[undoManager prepareWithInvocationTarget:self] resetWithX: xPos y: yPos width: width height: height];    
    // then set undo values and rebuild transformation
    xPos = x;
    yPos = y;
    width = w;
    height = h;
}

- (void) adaptPaths
{
	// associatedPaths
	NSEnumerator *enumerator = [associatedPaths objectEnumerator];
	OCEPathObject *object;
	
	while (object = [enumerator nextObject]) 
	{
		[object adaptPathToRects];
	}	
}

- (void) adaptWithOffset: (NSRect) rect
{
	xPos -= rect.origin.x;
	yPos -= rect.origin.y;
	[self reset];
}

// ----------------------------------------------------------------------
// NSCopying Protocol
// ----------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
{
    OCEActiveComponent *copy = [super copyWithZone: zone];
    
	copy->color = [color retain];

    return copy;
}

// ----------------------------------------------------------------------
// save stuff
// ----------------------------------------------------------------------
- (NSMutableArray *)dataRepresentation
{
	NSMutableArray *array = [[super dataRepresentation] retain];
	
	return [array autorelease];
}

- (id)initWithContentsOfArray:(NSArray *) array forDocument: (OCEDocument *) newDocument
{
    [self init];
	
    return [super initWithContentsOfArray: array forDocument: newDocument];
}

- (NSString *) exportFlash {
	NSMutableString *data = [NSMutableString stringWithFormat:@"<hotspot x=\"%.0f\" y=\"%.0f\" w=\"%.0f\" h=\"%.0f\">\n", xPos,yPos,width,height];
		
	// trails
	unsigned i,c = [associatedPaths count];
	for (i=0; i<c; i++) {
		if ([[[associatedPaths objectAtIndex:i] associatedStartRect] rectID] == rectID){
			[data appendString: [[associatedPaths objectAtIndex:i] exportFlash]];
		}
	}
	
	//[super exportFlash: data];
	[data appendString: @"</hotspot>\n"];
	return data;
}


// ----------------------------------------------------------------------
// OTHER
// ----------------------------------------------------------------------
- (bool) isActiveComponent;
{
	return YES;	
}


@end
