//
//  OCEPanelView.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCEPanelView.h"
#import "OCEDocument.h"
#import "OCEWindowController.h"
#import "OCEPanel.h"
#import "OCEPanelObject.h"
#import "OCECanvas.h"

@implementation OCEPanelView

- (id)initWithFrame:(NSRect)frameRect
{
	[super initWithFrame:frameRect];
        if (self) 
        {
            dragging = NO;
            selectedObjects = [[NSMutableArray arrayWithCapacity:0] retain];

            // welche filetypen wollen wir?
            //[self registerForDraggedTypes:[NSArray arrayWithObjects: NSFilenamesPboardType, nil]];
            
            nc = [NSNotificationCenter defaultCenter];
            [nc addObserver: self selector: @selector(updateView) name: @"panelsUpdated" object: nil];
        }
	return self;
}

- (void) updateView
{
    [self setNeedsDisplay:YES];
}

// Methoden fr direkten Aufruf
- (void)setWindowController:(OCEWindowController *)aController {
    controller = aController;
    panelModel = [[self panelWindowController] document];
}

- (OCEWindowController *)panelWindowController {
    return controller;
}

- (OCEDocument *)panelModel {
    return [[self panelWindowController] document];
}

- (void)drawRect:(NSRect)rect
{
    // Groesse des views
    int count = [[self panelModel] panelCount];

    // berechnet wieviele pixel der view gross sein muss
    int groesse = ((int) ceil((double) count/2.0) * (cPreviewImageHeight+cPreviewImageSpacing));     
    int scrollViewHeight = [[self enclosingScrollView] documentVisibleRect].size.height;
    if (groesse < scrollViewHeight) 
        groesse = scrollViewHeight;

    [self setFrameSize:NSMakeSize([self frame].size.width,groesse)];

    // hintergrund
    [[NSColor grayColor] set];
    NSRectFill(rect);

    NSEnumerator *imageEnumerator = [[[self panelModel] getPanels] objectEnumerator];
    OCEPanel *image;
    int x = cPreviewImageSpacing;
    int y = cPreviewImageSpacing;
    
    // alle bilder anzeigen
    while(image = [imageEnumerator nextObject]) 
    { 
        NSRect aRect;
        if (NSIntersectsRect(aRect=NSMakeRect(x,y,cPreviewImageWidth,cPreviewImageHeight), rect)) 
        {
            // muss hier gesetzt werden weil farbe von drawInPanelView:isSelected ueber schrieben werden kann
            [[NSColor darkGrayColor] set];
            NSRectFill(NSIntersectionRect(aRect, rect)); // bild hintergrund
        }

        if (NSIntersectsRect([image panelViewRect], rect)) 
        {
            if ([selectedObjects containsObject: image])
                [image drawInPanelView: rect isSelected:YES];
            else 
                [image drawInPanelView:rect isSelected:NO];            
        }
        
        if (x==cPreviewImageSpacing) 
        {
            x=x+cPreviewImageWidth+cPreviewImageSpacing;
        }
        else 
        {
            x=cPreviewImageSpacing;
            y=y+cPreviewImageHeight+cPreviewImageSpacing;
        }
    }
    
    // beim ziehen eines filenames
    // erst hier damit es das oberste element ist
    if (dragging) 
    {
        [[NSColor darkGrayColor] set];
        [NSBezierPath setDefaultLineWidth: 5];
        [NSBezierPath strokeRect: rect];
    }
}


// if "Import ..."-Button is pressed, this method ist called
// it opens an Open-Sheet
- (IBAction)openImportPanel:(id)sender 
{
    NSOpenPanel *panel = [NSOpenPanel openPanel];
    [panel setAllowsMultipleSelection:YES]; // i want to select multible files
    [panel beginSheetForDirectory:nil 
							 file:nil 
							types:[NSArray arrayWithObjects: @"gif", @"jpg", @"jpeg", @"png", nil]
				   modalForWindow:mainWindow 
					modalDelegate:self 
				   didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:) 
					  contextInfo: nil];
}

// OpenPanel wurde beendet
// jetzt werden alle Bilder geladen und in einem NSMutableArray gespeichert
- (void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo 
{
    if (returnCode == NSOKButton) 
    {
        [[self panelModel] loadImagesFromPaths:[sheet filenames]];
		[self setPanelCountDisplay];
		[self setToolTips];
        [self setNeedsDisplay:YES];
    }
}


// frhlingsputz
- (void)dealloc 
{
    [self unregisterDraggedTypes];
    [selectedObjects release];
    [super dealloc];
}

// erhht die performance
- (BOOL)isOpaque 
{
    return YES; 
}

// ich verwenden gespiegelte koordination
// ursprung ist obere linke ecke
- (BOOL)isFlipped 
{
    return YES;
}

// wenn man auf den view klickt bewegt sich nicht das fenster mit
- (BOOL)mouseDownCanMoveWindow 
{
    return NO;
}

// i want to respond to events
- (BOOL)acceptsFirstResponder {
    return YES;
}

// ----------------------------------------------------------------------
// DRAG DESTINATION METHODS
// ----------------------------------------------------------------------
// curser setzen
- (NSDragOperation)draggingEntered:(id <NSDraggingInfo>)sender {
    /*NSPasteboard *pboard;
    NSDragOperation sourceDragMask;

    sourceDragMask = [sender draggingSourceOperationMask];
    pboard = [sender draggingPasteboard];

    if ( [[pboard types] containsObject:NSFilenamesPboardType] ) {
        if (sourceDragMask & NSDragOperationLink) {
            dragging = YES;
            [self setNeedsDisplay:YES];
            return NSDragOperationLink;
        } else if (sourceDragMask & NSDragOperationCopy) {
            dragging = YES;
            [self setNeedsDisplay:YES];
            return NSDragOperationCopy;
        }
    }*/
    return NSDragOperationNone;
}

- (void)draggingExited:(id <NSDraggingInfo>)sender
{
    dragging = NO;
    [self setNeedsDisplay:YES];
}

// ja, ich bin bereit :)
- (BOOL)prepareForDragOperation:(id <NSDraggingInfo>)sender
{
    dragging=NO;
    return YES;
}

// drag operation durchfhren 
- (BOOL)performDragOperation:(id <NSDraggingInfo>)sender
{
    NSPasteboard *pasteBoard = [sender draggingPasteboard];

    if ( [[pasteBoard types] containsObject:NSFilenamesPboardType] ) 
    {
        NSArray *files = [pasteBoard propertyListForType:NSFilenamesPboardType];
        bool returnvalue = [[self panelModel] loadImagesFromPaths:files];
        [self setNeedsDisplay:YES];
        return returnvalue;
    }
    else
    {
        return NO; // keine passenden filetypen gefunden
    }

}

- (void)concludeDragOperation:(id <NSDraggingInfo>)sender
{
    [self setNeedsDisplay:YES];
}

// ----------------------------------------------------------------------
// DRAG SOURCE METHODS
// ----------------------------------------------------------------------

// "- (void)mouseDragged:(NSEvent *)theEvent" erledigt den grossteil der Arbeit!!!!
- (unsigned int)draggingSourceOperationMaskForLocal:(BOOL)isLocal
{
    if (isLocal) // destination ist in der selben application
        return NSDragOperationLink;
    else 
        return NSDragOperationNone;
}

- (BOOL)ignoreModifierKeysWhileDragging
{
    return YES;
}

// ----------------------------------------------------------------------
// EVENT HANDLE METHODS
// ----------------------------------------------------------------------
- (void)mouseDown:(NSEvent *)theEvent // handels user input (main mouse button pressed)
{
    NSEnumerator *imageEnumerator = [[[self panelModel] getPanels] objectEnumerator];
    OCEPanel *image;
    NSPoint mouseLocation = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    while(image = [imageEnumerator nextObject]) 
    {
        if ([self mouse:mouseLocation inRect:[image panelViewRect]]) 
        {
            if ([selectedObjects containsObject:image])
            {
                if ([theEvent modifierFlags] & NSCommandKeyMask)
                    [selectedObjects removeObject:image];
            }
            else
            {
                if (!([theEvent modifierFlags] & NSCommandKeyMask))
                    [selectedObjects removeAllObjects];
                [selectedObjects addObject: image];
            }
            break;
        }
    }
    if (!image)
    {
        if (!([theEvent modifierFlags] & NSCommandKeyMask))
            [selectedObjects removeAllObjects];
    }
    
    [self setNeedsDisplay:YES];
}

- (void)mouseDragged:(NSEvent *)theEvent 
// Erledigt Aufgabe fr die Drag Operation
{
    NSEnumerator *imageEnumerator = [selectedObjects objectEnumerator];
    OCEPanel *selectedImage;
    NSPoint mouseLocation = [self convertPoint:[theEvent locationInWindow] fromView:nil];

    while(selectedImage = [imageEnumerator nextObject]) 
    {
        if ([self mouse:mouseLocation inRect:[selectedImage panelViewRect]]) 
        {
        
            NSPasteboard *pboard;
            pboard = [NSPasteboard pasteboardWithName:NSDragPboard];
        
            // store the pointer in a NSData Object ..
            int *panelAddress = (void *) &selectedObjects;

            [pboard declareTypes:[NSArray arrayWithObject:OCEDragType] owner:self];
            NSData *data = [NSArchiver archivedDataWithRootObject:[NSNumber numberWithInt: *panelAddress]];
            [pboard setData:data forType:OCEDragType];
            
            //dragImage durchsichtig machen
            NSSize size = [[selectedImage image] size];
            if (size.width < 40)
                size.width = 40;
            if (size.height < 40)
                size.height = 40;
            NSImage* dragImage = [[NSImage alloc] initWithSize:size];
            [dragImage lockFocus]; 
            [[selectedImage image] dissolveToPoint: NSMakePoint(0,0) fraction: .5];
			
			// anzahl der gezogenen bilder
            int count = [selectedObjects count];
            if (count > 1)
            {
				NSString *imagePath = [[NSBundle mainBundle] pathForResource:@"panelCountImage" ofType:@"tiff"];
				NSImage *panelCountImage = [[NSImage alloc] initWithContentsOfFile: imagePath];
				if (panelCountImage) 
				{
					[panelCountImage drawInRect: NSMakeRect(size.width-23,3, 20,20)
									   fromRect: (NSRect) {{0,0},[panelCountImage size]}
									  operation: NSCompositeSourceOver
									   fraction: 1];
				}
                [[NSColor whiteColor] set];
                NSString *string = [[NSNumber numberWithInt:count] stringValue];
                if (count < 10)
                    [string drawInRect: (NSRect) {{size.width-16,7},{7,14}} 
						withAttributes: [NSDictionary dictionaryWithObject: [NSColor whiteColor] 
																	forKey: NSForegroundColorAttributeName]];
                else
                    [string drawInRect: (NSRect) {{size.width-20,7},{14,14}}
						withAttributes: [NSDictionary dictionaryWithObject: [NSColor whiteColor]
																	forKey: NSForegroundColorAttributeName]];
            }
            [dragImage unlockFocus];
            
            // Position des draggedImage berechnen
            NSRect imageRect = [selectedImage panelViewRect];
            float canvasScale = [[controller canvas] scaleCanvasFactor];
            
            // scale dragImage if canvas is scaled
            [dragImage setScalesWhenResized:YES];
            NSSize dragImageSize = [dragImage size];
            dragImageSize.width = dragImageSize.width / canvasScale;
            dragImageSize.height = dragImageSize.height / canvasScale;
            [dragImage setSize:dragImageSize];
            
            float imageScale = [[selectedImage image] size].width / imageRect.size.width;
            float xOffset = ((mouseLocation.x-imageRect.origin.x) * imageScale) / canvasScale;
            float yOffset = ((imageRect.origin.y+imageRect.size.height-mouseLocation.y)) * imageScale / canvasScale;

            NSPoint dragImageOrigin = mouseLocation;
            dragImageOrigin.x = dragImageOrigin.x-xOffset;
            dragImageOrigin.y = dragImageOrigin.y+yOffset;

            [selectedObjects removeObject: selectedImage];
            [selectedObjects insertObject: selectedImage atIndex:0];
            
            [self dragImage: dragImage //image to be displayed under the mouse
                at:  dragImageOrigin //point to start drawing drag image
                offset: NSZeroSize //no offset, drag starts at mousedown location
                event:theEvent
                pasteboard:pboard
                source: self
                slideBack: YES];
            break; // exit while
        }
    }
}

- (void)keyDown:(NSEvent *)theEvent
{
        [self interpretKeyEvents:[NSArray arrayWithObject:theEvent]];
}

- (void)deleteForward:(id)sender
// wird beim drcken von entf ausgefhrt. hat ursprung in NSResponder
{
    NSEnumerator *imageEnumerator = [selectedObjects objectEnumerator];
    OCEPanel *image;
	bool exists = NO;
	
    while(image = [imageEnumerator nextObject]) 
    {
        if ([panelModel  panelExistsOnCanvas: image])
        {
			exists = YES;
        }
        else
        {
            [[self panelModel] removePanel:image];
            [selectedObjects removeObject:image];
        }
    }
	if (exists)
		NSRunAlertPanel(@"Warning!", @"One or more Panels couldn't be deleted, because they exist on the Canvas.", 
						@"Continue", nil,nil);
	[self setPanelCountDisplay];
	[self setToolTips];
    [self setNeedsDisplay:YES];
}

- (void)deleteBackward:(id)sender
{
    [self deleteForward:sender];
}

- (IBAction)delete:(id)sender
{
    [self deleteForward: sender];
}

- (IBAction)selectAll:(id)sender
{
    [selectedObjects removeAllObjects];
    NSEnumerator *objectsEnumerator = [[[self panelModel] getPanels] objectEnumerator];
    OCEPanel *object;
    while(object = [objectsEnumerator nextObject]) 
    {
        [selectedObjects addObject: object];
    }
    [self setNeedsDisplay:YES];
}

- (IBAction)selectVisible:(id)sender
{
	[self clearSelection];
	
	OCEPanel *object;
	unsigned int i = 0;
	NSArray *objects = [[self panelModel] getPanels];
	while (i < [objects count]) 
	{
		object = [objects objectAtIndex:i];
		if (NSIntersectsRect([object panelViewRect],[[self enclosingScrollView] documentVisibleRect]))
			[self selectObject:object];
		i++;
	}
	[self setNeedsDisplay:YES];	
}

- (IBAction)selectNone:(id)sender
{
	[self clearSelection];
	[self setNeedsDisplay:YES];	
}

// ----------------------------------------------------------------------
// Select Methoden
// ----------------------------------------------------------------------
- (NSMutableArray *) selectedObjects
{
    return selectedObjects;
}

- (BOOL)objectIsSelected:(OCEPanel *)object 
{
    return (([selectedObjects indexOfObjectIdenticalTo:object] == NSNotFound) ? NO : YES);
}

- (void)selectObject:(OCEPanel *)object {
    unsigned curIndex = [selectedObjects indexOfObjectIdenticalTo:object];
    if (curIndex == NSNotFound) 
    {
        [selectedObjects addObject:object];
    }
}

- (void)deselectObject:(OCEPanel *)object 
{
    unsigned curIndex = [selectedObjects indexOfObjectIdenticalTo:object];
    if (curIndex != NSNotFound) 
    {
        [selectedObjects removeObjectAtIndex:curIndex];
    }
}

- (void)clearSelection 
{
    [selectedObjects removeAllObjects];
	[self setNeedsDisplay:YES];
}


- (void) setPanelCountDisplay
{
    // set countDisplay
	int count = [[self panelModel] panelCount];
    if ((count == 1) || (count == 0)) 
        [countDisplay setStringValue:[NSString stringWithFormat:@"%i Panel",count]];
    else 
        [countDisplay setStringValue:[NSString stringWithFormat:@"%i Panels",count]];
}

// tool tips
- (void) setToolTips {
	[self removeAllToolTips];
	
	NSEnumerator *panelEnumerator = [[[self panelModel] getPanels] objectEnumerator];
    OCEPanel *panel;
	
    while (panel = [panelEnumerator nextObject]) {
		[self addToolTipRect: [panel panelViewRect] owner: [panel panelViewToolTip] userData: nil];
		NSLog([panel panelViewToolTip]);
	}
}

@end
