//
//  OCEPathObject.h
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import <Foundation/Foundation.h>
#import "OCEObject.h"
@class OCERectObject;

@interface OCEPathObject : OCEObject {
    NSBezierPath *path;
    NSColor *color;
    float thickness;
    OCERectObject *startRect;
    OCERectObject *endRect;
	int arrow;
	float arrowRotation1;
    float arrowRotation2;
	bool allowFreeEndPoints;
	
	int startRectID;
	int endRectID;
}

// ----------------------------------------------------------------------
// Draw
// ----------------------------------------------------------------------
- (void) drawKnobs;

// ----------------------------------------------------------------------
// arrow
// ----------------------------------------------------------------------
- (void) calculateArrowRotation;
- (void) drawArrows;


// ----------------------------------------------------------------------
// Attributes
// ----------------------------------------------------------------------
- (int) arrow;
- (void) setArrow: (int) newArrow; 

- (NSColor *) color;
- (void) setColor: (NSColor *) newColor;
- (float) thickness;
- (void) setThickness: (float) newThickness;
- (void) setKnob: (enum corner) knob toPoint: (NSPoint) point;
- (NSPoint) startPoint;
- (void) setStartPoint: (NSPoint) point;
- (NSPoint) endPoint;
- (void) setEndPoint: (NSPoint) point;
- (bool) endPointAtKnob: (int) knob;
- (void) changeSegment: (int) segment;
- (bool) valid;


// ----------------------------------------------------------------------
// Create
// ----------------------------------------------------------------------
- (id) initWithPoint: (NSPoint) point;
- (void) addLineToPoint: (NSPoint) point;
- (void) addCurveToPoint: (NSPoint) point controlPoint1: (NSPoint) controlPoint1 controlPoint2: (NSPoint) controlPoint2;

// ----------------------------------------------------------------------
// Associated Rects
// ----------------------------------------------------------------------
- (OCERectObject *) associatedStartRect;
- (void) setAssociatedStartRect: (OCERectObject *) rectObject;
- (OCERectObject *) associatedEndRect;
- (void) setAssociatedEndRect: (OCERectObject *) rectObject;
- (void) unregisterAssociatedStartRect;
- (void) unregisterAssociatedEndRect;
- (void) unregisterAssociatedRects;
- (void) unregisterAssociatedRect: (OCERectObject *) rectObject;
- (void) adaptPathToRects;

// ----------------------------------------------------------------------
// Save/Load
// ----------------------------------------------------------------------
- (void) finishLoadingObject: (OCEDocument *) document;
- (void) exportFlash: (NSMutableString *) data;

@end
