//
//  OCENavFrameObject.h
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import <Foundation/Foundation.h>
#import "OCEObject.h";
@class OCEPathObject;


@interface  OCERectObject: OCEObject {
    double xPos, yPos, width, height;
    float rotation;
	float proportion;
	NSRect offsetRect;
    NSAffineTransform *transform;
    NSMutableArray *associatedPaths; 
    double saveUndoValuesArray[5];
    bool isSquare;
	
	int rectID;
}

// init Methods
- (id) initWithRect: (NSRect) aRect;

// draw Methods
- (void) drawKnobs;
- (void) drawFocusRing;

// manipulate
- (void) rotateKnob: (enum corner)knob fromPoint: (NSPoint)lastPoint toPoint: (NSPoint)point stepwise: (bool)flag;
- (NSRect) resizeKnob: (enum corner)knob xOffset: (float)xOffset yOffset: (float)yOffset;
- (NSRect) resizeAroundCenterKnob: (enum corner)knob xOffset: (float)xOffset yOffset: (float)yOffset;
- (NSRect) resizeWithFixedProportionsKnob: (enum corner *)knob xOffset: (float)xOffset yOffset: (float)yOffset;
- (void) reset;
- (void) setOrigin: (NSPoint) aPoint;
- (OCERectObject *) checkIfFocusRingObject: (NSPoint)point isStartPoint:(bool) startPoint;


// set Methods
- (void) resizeTo: (NSRect) rect;
- (float) rotation;
- (void) setRotation: (float) newRotation;
- (void) resetWithX: (double) x y: (double) y width: (double) w height: (double) h rotation: (float) r;
- (bool) isSquare;
- (double) width;
- (void) setWidth: (double) w;
- (double) height;
- (void) setHeight: (double) h;
- (double) originX;
- (void) setOriginX: (double) x;
- (double) originY;
- (void) setOriginY: (double) y;
- (NSPoint) center;

// associated paths
- (void) addAssociatedPath: (OCEPathObject *) path; // doesn't register the trail
- (void) removeAssociatedPath: (OCEPathObject *) path;  // doesn't unregister the trail
- (void) unregisterAssociatedPaths;
- (NSArray *) outgoingAssociatedPaths;

// ID
- (int) rectID;
- (void) setRectID: (int) newID;
- (OCERectObject *) rectObjectFromID: (int) anID;

// other
- (bool) isActiveComponent;


@end
