//
//  OCETrailObject.m
//  InfiniteCanvas
//
//  Created by Markus Mueller <markus@mindnode.com>.
//	http://www.infinitecanvas.com/
//  Copyright (c) 2003-2005 Markus Mueller. All rights reserved.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//  

#import "OCETrailObject.h"
#import "OCECanvas.h"
#import "OCEPanelObject.h"
#import "OCEDocument.h"
#import "NSMutableArrayDeepCopy.h"
#import "NSBezierPathHitTest.h"
#import "OCEMath.h"
#import "ICFlashConverter.h"

@implementation OCETrailObject


// ----------------------------------------------------------------------
// from super class
// ----------------------------------------------------------------------
- (id)init 
{
    [super init];

    arrow = 0;
    dashStyle = 1; // damit ist array gesetzt wird
    [self setDashStyle: 0];
	
	[self setSnapAble:YES];
	
    return self;
}

- (void)dealloc 
{
	//if (arrowImage)
	//	[arrowImage release];
    [super dealloc];
}
// ----------------------------------------------------------------------
// Arrow
// ----------------------------------------------------------------------
- (void) drawArrows
{
	NSAffineTransform *transform;
	NSPoint point;
	NSGraphicsContext *currentContext = [NSGraphicsContext currentContext];
	
	if (arrow < 4)
	{
		if ((arrow == 1) || (arrow == 3))
		{
			point = [self endPoint];
			transform = [NSAffineTransform transform];
			[transform translateXBy: point.x yBy: point.y];
			[transform rotateByDegrees: arrowRotation1];
			[transform translateXBy: -point.x yBy: -point.y];
			
			NSBezierPath *arrowPath = [NSBezierPath bezierPath];                  
			[arrowPath moveToPoint:NSMakePoint(point.x-10,point.y+10)];
			[arrowPath lineToPoint:point];
			[arrowPath lineToPoint:NSMakePoint(point.x-10,point.y-10)];
			[arrowPath setLineWidth:thickness];			
			
			[currentContext saveGraphicsState];
			[transform concat];
			[arrowPath stroke];
			[currentContext restoreGraphicsState];
		}
		if ((arrow == 2) || (arrow == 3))
		{
			point = [self startPoint];
			transform = [NSAffineTransform transform];
			[transform translateXBy: point.x yBy: point.y];
			[transform rotateByDegrees: arrowRotation2];
			[transform translateXBy: -point.x yBy: -point.y];
			
			NSBezierPath *arrowPath = [NSBezierPath bezierPath];                  
			[arrowPath moveToPoint:NSMakePoint(point.x-10,point.y+10)];
			[arrowPath lineToPoint:point];
			[arrowPath lineToPoint:NSMakePoint(point.x-10,point.y-10)];
			[arrowPath setLineWidth:thickness];
			
			[currentContext saveGraphicsState];
			[transform concat];
			[arrowPath stroke];
			[currentContext restoreGraphicsState];
		}
	}
	else
	{
		if ((arrow == 4) || (arrow == 6))
		{
			point = [self endPoint];
			transform = [NSAffineTransform transform];
			[transform translateXBy: point.x yBy: point.y];
			[transform rotateByDegrees: arrowRotation1];
			[transform translateXBy: -point.x yBy: -point.y];
			
			NSBezierPath *arrowPath = [NSBezierPath bezierPath];                  
            [arrowPath moveToPoint:NSMakePoint(point.x-15,point.y+7)];
            [arrowPath lineToPoint:point];
            [arrowPath lineToPoint:NSMakePoint(point.x-15,point.y-7)];
			[arrowPath moveToPoint:NSMakePoint(point.x-15,point.y+7)]; // workaround
            [arrowPath lineToPoint:NSMakePoint(point.x-5,point.y)];
            [arrowPath lineToPoint:NSMakePoint(point.x-15,point.y-7)];			
			[arrowPath setLineWidth:thickness];

			[currentContext saveGraphicsState];
			[transform concat];
			[arrowPath stroke];
			[currentContext restoreGraphicsState];
		}
		if ((arrow == 5) || (arrow == 6))
		{
			point = [self startPoint];
			transform = [NSAffineTransform transform];
			[transform translateXBy: point.x yBy: point.y];
			[transform rotateByDegrees: arrowRotation2];
			[transform translateXBy: -point.x yBy: -point.y];
			
			NSBezierPath *arrowPath = [NSBezierPath bezierPath];                  
            [arrowPath moveToPoint:NSMakePoint(point.x-15,point.y+7)];
            [arrowPath lineToPoint:point];
            [arrowPath lineToPoint:NSMakePoint(point.x-15,point.y-7)];
			[arrowPath moveToPoint:NSMakePoint(point.x-15,point.y+7)]; // workaround
            [arrowPath lineToPoint:NSMakePoint(point.x-5,point.y)];
            [arrowPath lineToPoint:NSMakePoint(point.x-15,point.y-7)];			
			[arrowPath setLineWidth:thickness];
			
			[currentContext saveGraphicsState];
			[transform concat];
			[arrowPath stroke];
			[currentContext restoreGraphicsState];
		}
		
	}
}

// ----------------------------------------------------------------------
// Attributes
// ----------------------------------------------------------------------
/*- (void) setArrowImage: (NSImage *) newImage
{
    id oldImage = arrowImage;
    arrowImage = [newImage retain];
    [oldImage release];
}*/

- (void) setDashStyle: (int) style
{
    if (dashStyle != style)
    {
        [[undoManager prepareWithInvocationTarget:self] setDashStyle: dashStyle];
		[undoManager setActionName: @"Change Dash Style"];

        dashStyle = style;
		[self applyDashStyle];
    }
}

- (int) dashStyle
{
    return dashStyle;
}

- (void) applyDashStyle
{
	if (!path)
		return;
	
	float dashArray[3];

	switch (dashStyle)
	{
		case 0:
			[path setLineDash:dashArray count:0 phase: 0.0];
			break;
		case 1:
			dashArray[0] = 5;
			dashArray[1] = 5;
			[path setLineDash:dashArray count:2 phase: 0.0];
			break;
		case 2:
			dashArray[0] = 8;
			dashArray[1] = 3;
			dashArray[2] = 8;
			[path setLineDash:dashArray count:3 phase: 0.0];
			break;
		case 3:
			dashArray[0] = 9;
			dashArray[1] = 6;
			dashArray[2] = 3;
			[path setLineDash:dashArray count:3 phase: 0.0];
			break;
	}
}

- (void) changeSegment: (int) segment
{
	[super changeSegment:segment];
	[self applyDashStyle];
}

// ----------------------------------------------------------------------
// NSCopying Protocol
// ----------------------------------------------------------------------
- (id)copyWithZone:(NSZone *)zone
{
    OCETrailObject *copy = [super copyWithZone: zone];
    //if (arrowImage)
	//	copy->arrowImage = [arrowImage retain];
    copy->dashStyle = dashStyle;
    
    return copy;
}

// ----------------------------------------------------------------------
// Save and Load
// ----------------------------------------------------------------------
- (NSMutableArray *) dataRepresentation
{
	NSMutableArray *array = [[super dataRepresentation] retain];
	
	[array addObject:[NSNumber numberWithInt:dashStyle]];
	[array addObject:[NSNumber numberWithFloat:arrowRotation1]]; // wird gespeichert um mir die berechnung im applet
	[array addObject:[NSNumber numberWithFloat:arrowRotation2]]; // zu ersparen
	
	return [array autorelease];
}

- (id)initWithContentsOfArray:(NSArray *) array forDocument: (OCEDocument *) newDocument
{
    [self init];
    dashStyle = [[array objectAtIndex:10] intValue];
	
	id  returnValue = [super initWithContentsOfArray: array forDocument: newDocument];
	[self applyDashStyle];

    return returnValue;
}

- (NSString *) exportFlash;
{
	// color stuff
	NSColor *tmpColor = color;
	if (![[tmpColor colorSpaceName] isEqualToString:@"NSCalibratedRGBColorSpace"])
		if ((tmpColor = [tmpColor colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"]) == nil)
			tmpColor = [[NSColor blackColor] colorUsingColorSpaceName: @"NSCalibratedRGBColorSpace"];
	int bgR = [tmpColor redComponent] * 255;
	int bgG = [tmpColor greenComponent] * 255;
	int bgB = [tmpColor blueComponent] * 255;
	

	// creater string
	NSMutableString *data = [NSMutableString stringWithFormat:@"<trail x=\"%.0f\" y=\"%.0f\" thickness=\"%.1f\" color=\"0x%02x%02x%02x\">\n", [self startPoint].x,[self startPoint].y*-1,thickness,bgR,bgG,bgB];
	[super exportFlash: data];
	[data appendString:@"</trail>\n"];
	return data;
}

@end
